/*
    mtr  --  a network diagnostic tool
    Copyright (C) 1997  Matt Kimball

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <config.h>
#include <unistd.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>

#define HostSize 256
#define NameSize 256

pid_t dns_pid;
int requestpipes[2];
int resultpipes[2];

char Names[HostSize][NameSize];
int Addresses[HostSize];
int AddressesUsed;

#define CommandQuit 0x80
#define CommandLookup 0x81
#define CommandCompleted 0x82

int dns_forward(char *name) {
  struct hostent *host;

  host = gethostbyname(name);
  if(host == NULL)
    return 0;

  return *(int *)host->h_addr;
}

char *dns_lookup(int address) {
  int at;
  int command;

  for(at = 0; at < AddressesUsed; at++) {
    if(Addresses[at] == address) {
      if(strlen(Names[at]) > 0) 
	return Names[at];
      else
	return NULL;
    }
  }

  if(AddressesUsed >= HostSize)
    return NULL;

  command = CommandLookup;
  write(requestpipes[1], &command, sizeof(int));
  write(requestpipes[1], &address, sizeof(int));

  Addresses[AddressesUsed] = address;
  strcpy(Names[AddressesUsed], "");
  AddressesUsed++;

  return NULL;
}

int dns_loop() {
  int command;
  int address;
  char name[NameSize];
  struct hostent *host;

  read(requestpipes[0], &command, sizeof(int));
  if(command == CommandQuit)
    return 1;
  if(command == CommandLookup) {
    read(requestpipes[0], &address, sizeof(int));
    address = htonl(address);
    host = gethostbyaddr((char *)&address, sizeof(int), AF_INET);
    if(host != NULL) {
      strncpy(name, host->h_name, NameSize);
      name[NameSize - 1] = 0;

      command = CommandCompleted;
      write(resultpipes[1], &command, sizeof(int));
      address = ntohl(address);
      write(resultpipes[1], &address, sizeof(int));
      write(resultpipes[1], name, NameSize);
    }
  }
  
  return 0;
}

int dns_waitfd() {
  return resultpipes[0];
}

void dns_open() {
  pipe(requestpipes);
  pipe(resultpipes);
  
  if(dns_pid == 0) {
    dns_pid = fork();
    if(dns_pid == 0) {
      while(!dns_loop());
      _exit(0);
    }
  }
}

void dns_ack() {
  int command, address, at;
  char name[NameSize];

  read(resultpipes[0], &command, sizeof(int));
  read(resultpipes[0], &address, sizeof(int));
  read(resultpipes[0], name, NameSize);
  name[NameSize - 1] = 0;

  for(at = 0; at < AddressesUsed; at++) {
    if(Addresses[at] == address) {
      strcpy(Names[at], name);
    }
  }
}

void dns_close() {
  int command;

  command = CommandQuit;
  write(requestpipes[1], &command, sizeof(int));

  kill(dns_pid, SIGKILL);
  waitpid(dns_pid, &command, 0);
}



